@@ -2,27 +2,29 @@
 
 #include "StdAfx.h"
 
-#include <stdio.h>
+#include "../../../../C/Alloc.h"
 
-#include "ZipUpdate.h"
-#include "ZipAddCommon.h"
-#include "ZipOut.h"
-
-#include "Common/Defs.h"
 #include "Common/AutoPtr.h"
+#include "Common/Defs.h"
 #include "Common/StringConvert.h"
+
 #include "Windows/Defs.h"
 #include "Windows/Thread.h"
 
-#include "../../Common/ProgressUtils.h"
-#ifdef COMPRESS_MT
-#include "../../Common/ProgressMt.h"
-#endif
+#include "../../Common/CreateCoder.h"
 #include "../../Common/LimitedStreams.h"
 #include "../../Common/OutMemStream.h"
-#include "../../Common/CreateCoder.h"
+#include "../../Common/ProgressUtils.h"
+#ifndef _7ZIP_ST
+#include "../../Common/ProgressMt.h"
+#endif
+#include "../../Common/StreamUtils.h"
 
-#include "../../Compress/Copy/CopyCoder.h"
+#include "../../Compress/CopyCoder.h"
+
+#include "ZipAddCommon.h"
+#include "ZipOut.h"
+#include "ZipUpdate.h"
 
 using namespace NWindows;
 using namespace NSynchronization;
@@ -30,27 +32,27 @@ using namespace NSynchronization;
 namespace NArchive {
 namespace NZip {
 
-#ifdef _WIN32
-static const Byte kMadeByHostOS = NFileHeader::NHostOS::kFAT;
-static const Byte kExtractHostOS = NFileHeader::NHostOS::kFAT;
-#else
-static const Byte kMadeByHostOS = NFileHeader::NHostOS::kUnix;
-static const Byte kExtractHostOS = NFileHeader::NHostOS::kUnix;
-#endif
+static const Byte kHostOS =
+  #ifdef _WIN32
+  NFileHeader::NHostOS::kFAT;
+  #else
+  NFileHeader::NHostOS::kUnix;
+  #endif
+
+static const Byte kMadeByHostOS = kHostOS;
+static const Byte kExtractHostOS = kHostOS;
 
 static const Byte kMethodForDirectory = NFileHeader::NCompressionMethod::kStored;
-static const Byte kExtractVersionForDirectory = NFileHeader::NCompressionMethod::kStoreExtractVersion;
 
-static HRESULT CopyBlockToArchive(ISequentialInStream *inStream, 
+static HRESULT CopyBlockToArchive(ISequentialInStream *inStream,
     COutArchive &outArchive, ICompressProgressInfo *progress)
 {
   CMyComPtr<ISequentialOutStream> outStream;
   outArchive.CreateStreamForCopying(&outStream);
-  CMyComPtr<ICompressCoder> copyCoder = new NCompress::CCopyCoder;
-  return copyCoder->Code(inStream, outStream, NULL, NULL, progress);
+  return NCompress::CopyStream(inStream, outStream, progress);
 }
 
-static HRESULT WriteRange(IInStream *inStream, COutArchive &outArchive, 
+static HRESULT WriteRange(IInStream *inStream, COutArchive &outArchive,
     const CUpdateRange &range, ICompressProgressInfo *progress)
 {
   UInt64 position;
@@ -66,30 +68,30 @@ static HRESULT WriteRange(IInStream *inStream, COutArchive &outArchive,
 }
 
 static void SetFileHeader(
-    COutArchive &archive, 
+    COutArchive &archive,
     const CCompressionMethodMode &options,
-    const CUpdateItem &updateItem, 
+    const CUpdateItem &ui,
     CItem &item)
 {
-  item.UnPackSize = updateItem.Size;
-  bool isDirectory;
+  item.UnPackSize = ui.Size;
+  bool isDir;
 
   item.ClearFlags();
 
-  if (updateItem.NewProperties)
+  if (ui.NewProperties)
   {
-    isDirectory = updateItem.IsDirectory;
-    item.Name = updateItem.Name; 
-    item.SetUtf8(updateItem.IsUtf8);
-    item.ExternalAttributes = updateItem.Attributes;
-    item.Time = updateItem.Time;
-    item.NtfsMTime = updateItem.NtfsMTime;
-    item.NtfsATime = updateItem.NtfsATime;
-    item.NtfsCTime = updateItem.NtfsCTime;
-    item.NtfsTimeIsDefined = updateItem.NtfsTimeIsDefined;
+    isDir = ui.IsDir;
+    item.Name = ui.Name;
+    item.SetUtf8(ui.IsUtf8);
+    item.ExternalAttributes = ui.Attributes;
+    item.Time = ui.Time;
+    item.NtfsMTime = ui.NtfsMTime;
+    item.NtfsATime = ui.NtfsATime;
+    item.NtfsCTime = ui.NtfsCTime;
+    item.NtfsTimeIsDefined = ui.NtfsTimeIsDefined;
   }
   else
-    isDirectory = item.IsDirectory();
+    isDir = item.IsDir();
 
   item.LocalHeaderPosition = archive.GetCurrentPosition();
   item.MadeByVersion.HostOS = kMadeByHostOS;
@@ -98,17 +100,17 @@ static void SetFileHeader(
   item.ExtractVersion.HostOS = kExtractHostOS;
 
   item.InternalAttributes = 0; // test it
-  item.SetEncrypted(!isDirectory && options.PasswordIsDefined);
-  if (isDirectory)
+  item.SetEncrypted(!isDir && options.PasswordIsDefined);
+  if (isDir)
   {
-    item.ExtractVersion.Version = kExtractVersionForDirectory;
+    item.ExtractVersion.Version = NFileHeader::NCompressionMethod::kExtractVersion_Dir;
     item.CompressionMethod = kMethodForDirectory;
     item.PackSize = 0;
     item.FileCRC = 0; // test it
   }
 }
 
-static void SetItemInfoFromCompressingResult(const CCompressingResult &compressingResult, 
+static void SetItemInfoFromCompressingResult(const CCompressingResult &compressingResult,
     bool isAesMode, Byte aesKeyMode, CItem &item)
 {
   item.ExtractVersion.Version = compressingResult.ExtractVersion;
@@ -134,7 +136,7 @@ static void SetItemInfoFromCompressingResult(const CCompressingResult &compressi
   }
 }
 
-#ifdef COMPRESS_MT
+#ifndef _7ZIP_ST
 
 static THREAD_FUNC_DECL CoderThread(void *threadCoderInfo);
 
@@ -201,7 +203,7 @@ void CThreadInfo::WaitAndCode()
       return;
     Result = Coder.Compress(
         #ifdef EXTERNAL_CODECS
-        _codecsInfo, _externalCodecs, 
+        _codecsInfo, _externalCodecs,
         #endif
         InStream, OutStream, Progress, CompressingResult);
     if (Result == S_OK && Progress)
@@ -294,7 +296,7 @@ HRESULT CMtProgressMixer2::SetRatioInfo(int index, const UInt64 *inSize, const U
     InSizes[index] = *inSize;
   if (outSize != 0)
     OutSizes[index] = *outSize;
-  UInt64 v = ProgressOffset + (_inSizeIsMain  ? 
+  UInt64 v = ProgressOffset + (_inSizeIsMain  ?
       (InSizes[0] + InSizes[1]) :
       (OutSizes[0] + OutSizes[1]));
   return Progress->SetCompleted(&v);
@@ -305,7 +307,7 @@ STDMETHODIMP CMtProgressMixer2::SetRatioInfo(const UInt64 *inSize, const UInt64
   return SetRatioInfo(0, inSize, outSize);
 }
 
-class CMtProgressMixer: 
+class CMtProgressMixer:
   public ICompressProgressInfo,
   public CMyUnknownImp
 {
@@ -333,14 +335,14 @@ STDMETHODIMP CMtProgressMixer::SetRatioInfo(const UInt64 *inSize, const UInt64 *
 #endif
 
 
-static HRESULT UpdateItemOldData(COutArchive &archive, 
+static HRESULT UpdateItemOldData(COutArchive &archive,
     IInStream *inStream,
-    const CUpdateItem &updateItem, CItemEx &item, 
+    const CUpdateItem &ui, CItemEx &item,
     /* bool izZip64, */
     ICompressProgressInfo *progress,
     UInt64 &complexity)
 {
-  if (updateItem.NewProperties)
+  if (ui.NewProperties)
   {
     if (item.HasDescriptor())
       return E_NOTIMPL;
@@ -349,15 +351,15 @@ static HRESULT UpdateItemOldData(COutArchive &archive,
     // CUpdateRange range(item.GetLocalExtraPosition(), item.LocalExtraSize + item.PackSize);
     CUpdateRange range(item.GetDataPosition(), item.PackSize);
     
-    // item.ExternalAttributes = updateItem.Attributes;
+    // item.ExternalAttributes = ui.Attributes;
     // Test it
-    item.Name = updateItem.Name; 
-    item.SetUtf8(updateItem.IsUtf8);
-    item.Time = updateItem.Time;
-    item.NtfsMTime = updateItem.NtfsMTime;
-    item.NtfsATime = updateItem.NtfsATime;
-    item.NtfsCTime = updateItem.NtfsCTime;
-    item.NtfsTimeIsDefined = updateItem.NtfsTimeIsDefined;
+    item.Name = ui.Name;
+    item.SetUtf8(ui.IsUtf8);
+    item.Time = ui.Time;
+    item.NtfsMTime = ui.NtfsMTime;
+    item.NtfsATime = ui.NtfsATime;
+    item.NtfsCTime = ui.NtfsCTime;
+    item.NtfsTimeIsDefined = ui.NtfsTimeIsDefined;
 
     item.CentralExtra.RemoveUnknownSubBlocks();
     item.LocalExtra.RemoveUnknownSubBlocks();
@@ -374,7 +376,7 @@ static HRESULT UpdateItemOldData(COutArchive &archive,
     CUpdateRange range(item.LocalHeaderPosition, item.GetLocalFullSize());
     
     // set new header position
-    item.LocalHeaderPosition = archive.GetCurrentPosition(); 
+    item.LocalHeaderPosition = archive.GetCurrentPosition();
     
     RINOK(WriteRange(inStream, archive, range, progress));
     complexity += range.Size;
@@ -384,22 +386,22 @@ static HRESULT UpdateItemOldData(COutArchive &archive,
 }
 
 static void WriteDirHeader(COutArchive &archive, const CCompressionMethodMode *options,
-    const CUpdateItem &updateItem, CItemEx &item)
+    const CUpdateItem &ui, CItemEx &item)
 {
-  SetFileHeader(archive, *options, updateItem, item);
-  archive.PrepareWriteCompressedData((UInt16)item.Name.Length(), updateItem.Size, options->IsAesMode);
+  SetFileHeader(archive, *options, ui, item);
+  archive.PrepareWriteCompressedData((UInt16)item.Name.Length(), ui.Size, options->IsAesMode);
   archive.WriteLocalHeader(item);
 }
 
 static HRESULT Update2St(
     DECL_EXTERNAL_CODECS_LOC_VARS
-    COutArchive &archive, 
+    COutArchive &archive,
     CInArchive *inArchive,
     IInStream *inStream,
     const CObjectVector<CItemEx> &inputItems,
     const CObjectVector<CUpdateItem> &updateItems,
-    const CCompressionMethodMode *options, 
-    const CByteBuffer &comment,
+    const CCompressionMethodMode *options,
+    const CByteBuffer *comment,
     IArchiveUpdateCallback *updateCallback)
 {
   CLocalProgress *lps = new CLocalProgress;
@@ -416,37 +418,37 @@ static HRESULT Update2St(
     lps->InSize = unpackSizeTotal;
     lps->OutSize = packSizeTotal;
     RINOK(lps->SetCur());
-    const CUpdateItem &updateItem = updateItems[itemIndex];
+    const CUpdateItem &ui = updateItems[itemIndex];
     CItemEx item;
-    if (!updateItem.NewProperties || !updateItem.NewData)
+    if (!ui.NewProperties || !ui.NewData)
     {
-      item = inputItems[updateItem.IndexInArchive];
+      item = inputItems[ui.IndexInArchive];
       if (inArchive->ReadLocalItemAfterCdItemFull(item) != S_OK)
         return E_NOTIMPL;
     }
 
-    if (updateItem.NewData)
+    if (ui.NewData)
     {
-      bool isDirectory = ((updateItem.NewProperties) ? updateItem.IsDirectory : item.IsDirectory());
-      if (isDirectory)
+      bool isDir = ((ui.NewProperties) ? ui.IsDir : item.IsDir());
+      if (isDir)
       {
-        WriteDirHeader(archive, options, updateItem, item);
+        WriteDirHeader(archive, options, ui, item);
       }
       else
       {
         CMyComPtr<ISequentialInStream> fileInStream;
-        HRESULT res = updateCallback->GetStream(updateItem.IndexInClient, &fileInStream);
+        HRESULT res = updateCallback->GetStream(ui.IndexInClient, &fileInStream);
         if (res == S_FALSE)
         {
-          lps->ProgressOffset += updateItem.Size;
+          lps->ProgressOffset += ui.Size;
           RINOK(updateCallback->SetOperationResult(NArchive::NUpdate::NOperationResult::kOK));
           continue;
         }
         RINOK(res);
 
         // file Size can be 64-bit !!!
-        SetFileHeader(archive, *options, updateItem, item);
-        archive.PrepareWriteCompressedData((UInt16)item.Name.Length(), updateItem.Size, options->IsAesMode);
+        SetFileHeader(archive, *options, ui, item);
+        archive.PrepareWriteCompressedData((UInt16)item.Name.Length(), ui.Size, options->IsAesMode);
         CCompressingResult compressingResult;
         CMyComPtr<IOutStream> outStream;
         archive.CreateStreamForCompressing(&outStream);
@@ -464,7 +466,7 @@ static HRESULT Update2St(
     {
       UInt64 complexity = 0;
       lps->SendRatio = false;
-      RINOK(UpdateItemOldData(archive, inStream, updateItem, item, progress, complexity));
+      RINOK(UpdateItemOldData(archive, inStream, ui, item, progress, complexity));
       lps->SendRatio = true;
       lps->ProgressOffset += complexity;
     }
@@ -477,13 +479,13 @@ static HRESULT Update2St(
 
 static HRESULT Update2(
     DECL_EXTERNAL_CODECS_LOC_VARS
-    COutArchive &archive, 
+    COutArchive &archive,
     CInArchive *inArchive,
     IInStream *inStream,
     const CObjectVector<CItemEx> &inputItems,
     const CObjectVector<CUpdateItem> &updateItems,
-    const CCompressionMethodMode *options, 
-    const CByteBuffer &comment,
+    const CCompressionMethodMode *options,
+    const CByteBuffer *comment,
     IArchiveUpdateCallback *updateCallback)
 {
   UInt64 complexity = 0;
@@ -493,20 +495,20 @@ static HRESULT Update2(
   int i;
   for(i = 0; i < updateItems.Size(); i++)
   {
-    const CUpdateItem &updateItem = updateItems[i];
-    if (updateItem.NewData)
+    const CUpdateItem &ui = updateItems[i];
+    if (ui.NewData)
     {
-      complexity += updateItem.Size;
-      numBytesToCompress += updateItem.Size;
+      complexity += ui.Size;
+      numBytesToCompress += ui.Size;
       numFilesToCompress++;
       /*
-      if (updateItem.Commented)
-        complexity += updateItem.CommentRange.Size;
+      if (ui.Commented)
+        complexity += ui.CommentRange.Size;
       */
     }
     else
     {
-      CItemEx inputItem = inputItems[updateItem.IndexInArchive];
+      CItemEx inputItem = inputItems[ui.IndexInArchive];
       if (inArchive->ReadLocalItemAfterCdItemFull(inputItem) != S_OK)
         return E_NOTIMPL;
       complexity += inputItem.GetLocalFullSize();
@@ -516,8 +518,8 @@ static HRESULT Update2(
     complexity += NFileHeader::kCentralBlockSize;
   }
 
-  if (comment != 0)
-    complexity += comment.GetCapacity();
+  if (comment)
+    complexity += comment->GetCapacity();
   complexity++; // end of central
   updateCallback->SetTotal(complexity);
 
@@ -525,7 +527,7 @@ static HRESULT Update2(
   
   complexity = 0;
   
-  #ifdef COMPRESS_MT
+  #ifndef _7ZIP_ST
 
   const size_t kNumMaxThreads = (1 << 10);
   UInt32 numThreads = options->NumThreads;
@@ -566,6 +568,14 @@ static HRESULT Update2(
       if (numThreads <= 1)
         mtMode = false;
     }
+    if (method == NFileHeader::NCompressionMethod::kLZMA)
+    {
+      UInt32 numLZMAThreads = (options->Algo > 0 ? 2 : 1);
+      numThreads /= numLZMAThreads;
+      options2.NumThreads = numLZMAThreads;
+      if (numThreads <= 1)
+        mtMode = false;
+    }
   }
 
   if (!mtMode)
@@ -576,7 +586,7 @@ static HRESULT Update2(
         inputItems, updateItems, options, comment, updateCallback);
 
 
-  #ifdef COMPRESS_MT
+  #ifndef _7ZIP_ST
 
   // Warning : before memManager, threads and compressingCompletedEvents
   // in order to have a "good" order for the destructor
@@ -584,6 +594,7 @@ static HRESULT Update2(
   synchroForCompressingCompletedEvents.Create();
   NWindows::NSynchronization::CSynchro synchroForOutStreamSpec;
   synchroForOutStreamSpec.Create();
+  
 
   CObjectVector<CItem> items;
 
@@ -592,7 +603,7 @@ static HRESULT Update2(
   mtProgressMixerSpec->Create(updateCallback, true);
 
   CMtCompressProgressMixer mtCompressProgressMixer;
-  mtCompressProgressMixer.Init(numThreads, mtProgressMixerSpec->RatioProgress); 
+  mtCompressProgressMixer.Init(numThreads, mtProgressMixerSpec->RatioProgress);
 
   CMemBlockManagerMt memManager(kBlockSize);
   CMemRefs refs(&memManager);
@@ -637,30 +648,30 @@ static HRESULT Update2(
   {
     if ((UInt32)threadIndices.Size() < numThreads && mtItemIndex < updateItems.Size())
     {
-      const CUpdateItem &updateItem = updateItems[mtItemIndex++];
-      if (!updateItem.NewData)
+      const CUpdateItem &ui = updateItems[mtItemIndex++];
+      if (!ui.NewData)
         continue;
       CItemEx item;
-      if (updateItem.NewProperties)
+      if (ui.NewProperties)
       {
-        if (updateItem.IsDirectory)
+        if (ui.IsDir)
           continue;
       }
       else
       {
-        item = inputItems[updateItem.IndexInArchive];
+        item = inputItems[ui.IndexInArchive];
         if (inArchive->ReadLocalItemAfterCdItemFull(item) != S_OK)
           return E_NOTIMPL;
-        if (item.IsDirectory())
+        if (item.IsDir())
           continue;
       }
       CMyComPtr<ISequentialInStream> fileInStream;
       {
         NWindows::NSynchronization::CCriticalSectionLock lock(mtProgressMixerSpec->Mixer2->CriticalSection);
-        HRESULT res = updateCallback->GetStream(updateItem.IndexInClient, &fileInStream);
+        HRESULT res = updateCallback->GetStream(ui.IndexInClient, &fileInStream);
         if (res == S_FALSE)
         {
-          complexity += updateItem.Size;
+          complexity += ui.Size;
           complexity += NFileHeader::kLocalBlockSize;
           mtProgressMixerSpec->Mixer2->SetProgressOffset(complexity);
           RINOK(updateCallback->SetOperationResult(NArchive::NUpdate::NOperationResult::kOK));
@@ -679,9 +690,9 @@ static HRESULT Update2(
           threadInfo.IsFree = false;
           threadInfo.InStream = fileInStream;
 
-          // !!!!! we must release ref before sending event 
+          // !!!!! we must release ref before sending event
           // BUG was here in v4.43 and v4.44. It could change ref counter in two threads in same time
-          fileInStream.Release(); 
+          fileInStream.Release();
 
           threadInfo.OutStreamSpec->Init();
           threadInfo.ProgressSpec->Reinit();
@@ -702,31 +713,31 @@ static HRESULT Update2(
       continue;
     }
 
-    const CUpdateItem &updateItem = updateItems[itemIndex];
+    const CUpdateItem &ui = updateItems[itemIndex];
 
     CItemEx item;
-    if (!updateItem.NewProperties || !updateItem.NewData)
+    if (!ui.NewProperties || !ui.NewData)
     {
-      item = inputItems[updateItem.IndexInArchive];
+      item = inputItems[ui.IndexInArchive];
       if (inArchive->ReadLocalItemAfterCdItemFull(item) != S_OK)
         return E_NOTIMPL;
     }
 
-    if (updateItem.NewData)
+    if (ui.NewData)
     {
-      bool isDirectory = ((updateItem.NewProperties) ? updateItem.IsDirectory : item.IsDirectory());
-      if (isDirectory)
+      bool isDir = ((ui.NewProperties) ? ui.IsDir : item.IsDir());
+      if (isDir)
       {
-        WriteDirHeader(archive, options, updateItem, item);
+        WriteDirHeader(archive, options, ui, item);
       }
       else
       {
         if (lastRealStreamItemIndex < itemIndex)
         {
           lastRealStreamItemIndex = itemIndex;
-          SetFileHeader(archive, *options, updateItem, item);
+          SetFileHeader(archive, *options, ui, item);
           // file Size can be 64-bit !!!
-          archive.PrepareWriteCompressedData((UInt16)item.Name.Length(), updateItem.Size, options->IsAesMode);
+          archive.PrepareWriteCompressedData((UInt16)item.Name.Length(), ui.Size, options->IsAesMode);
         }
 
         CMemBlocks2 &memRef = refs.Refs[itemIndex];
@@ -737,7 +748,7 @@ static HRESULT Update2(
           memRef.WriteToStream(memManager.GetBlockSize(), outStream);
           SetItemInfoFromCompressingResult(memRef.CompressingResult,
               options->IsAesMode, options->AesKeyMode, item);
-          SetFileHeader(archive, *options, updateItem, item);
+          SetFileHeader(archive, *options, ui, item);
           archive.WriteLocalHeader(item);
           // RINOK(updateCallback->SetOperationResult(NArchive::NUpdate::NOperationResult::kOK));
           memRef.FreeOpt(&memManager);
@@ -755,7 +766,7 @@ static HRESULT Update2(
             }
           }
 
-          DWORD result = ::WaitForMultipleObjects(compressingCompletedEvents.Size(), 
+          DWORD result = ::WaitForMultipleObjects(compressingCompletedEvents.Size(),
               &compressingCompletedEvents.Front(), FALSE, INFINITE);
           int t = (int)(result - WAIT_OBJECT_0);
           CThreadInfo &threadInfo = threads.Threads[threadIndices[t]];
@@ -768,9 +779,9 @@ static HRESULT Update2(
           {
             RINOK(threadInfo.OutStreamSpec->WriteToRealStream());
             threadInfo.OutStreamSpec->ReleaseOutStream();
-            SetItemInfoFromCompressingResult(threadInfo.CompressingResult, 
+            SetItemInfoFromCompressingResult(threadInfo.CompressingResult,
                 options->IsAesMode, options->AesKeyMode, item);
-            SetFileHeader(archive, *options, updateItem, item);
+            SetFileHeader(archive, *options, ui, item);
             archive.WriteLocalHeader(item);
           }
           else
@@ -786,7 +797,7 @@ static HRESULT Update2(
     }
     else
     {
-      RINOK(UpdateItemOldData(archive, inStream, updateItem, item, progress, complexity));
+      RINOK(UpdateItemOldData(archive, inStream, ui, item, progress, complexity));
     }
     items.Add(item);
     complexity += NFileHeader::kLocalBlockSize;
@@ -795,10 +806,220 @@ static HRESULT Update2(
   }
   archive.WriteCentralDir(items, comment);
   return S_OK;
-  #endif  
+  #endif
 }
 
-HRESULT Update(    
+static const size_t kCacheBlockSize = (1 << 20);
+static const size_t kCacheSize = (kCacheBlockSize << 2);
+static const size_t kCacheMask = (kCacheSize - 1);
+
+class CCacheOutStream:
+  public IOutStream,
+  public CMyUnknownImp
+{
+  CMyComPtr<IOutStream> _stream;
+  Byte *_cache;
+  UInt64 _virtPos;
+  UInt64 _virtSize;
+  UInt64 _phyPos;
+  UInt64 _phySize; // <= _virtSize
+  UInt64 _cachedPos; // (_cachedPos + _cachedSize) <= _virtSize
+  size_t _cachedSize;
+
+  HRESULT MyWrite(size_t size);
+  HRESULT MyWriteBlock()
+  {
+    return MyWrite(kCacheBlockSize - ((size_t)_cachedPos & (kCacheBlockSize - 1)));
+  }
+  HRESULT FlushCache();
+public:
+  CCacheOutStream(): _cache(0) {}
+  ~CCacheOutStream();
+  bool Allocate();
+  HRESULT Init(IOutStream *stream);
+  
+  MY_UNKNOWN_IMP
+
+  STDMETHOD(Write)(const void *data, UInt32 size, UInt32 *processedSize);
+  STDMETHOD(Seek)(Int64 offset, UInt32 seekOrigin, UInt64 *newPosition);
+  STDMETHOD(SetSize)(UInt64 newSize);
+};
+
+bool CCacheOutStream::Allocate()
+{
+  if (!_cache)
+    _cache = (Byte *)::MidAlloc(kCacheSize);
+  return (_cache != NULL);
+}
+
+HRESULT CCacheOutStream::Init(IOutStream *stream)
+{
+  _virtPos = _phyPos = 0;
+  _stream = stream;
+  RINOK(_stream->Seek(0, STREAM_SEEK_CUR, &_virtPos));
+  RINOK(_stream->Seek(0, STREAM_SEEK_END, &_virtSize));
+  RINOK(_stream->Seek(_virtPos, STREAM_SEEK_SET, &_virtPos));
+  _phyPos = _virtPos;
+  _phySize = _virtSize;
+  _cachedPos = 0;
+  _cachedSize = 0;
+  return S_OK;
+}
+
+HRESULT CCacheOutStream::MyWrite(size_t size)
+{
+  while (size != 0 && _cachedSize != 0)
+  {
+    if (_phyPos != _cachedPos)
+    {
+      RINOK(_stream->Seek(_cachedPos, STREAM_SEEK_SET, &_phyPos));
+    }
+    size_t pos = (size_t)_cachedPos & kCacheMask;
+    size_t curSize = MyMin(kCacheSize - pos, _cachedSize);
+    curSize = MyMin(curSize, size);
+    RINOK(WriteStream(_stream, _cache + pos, curSize));
+    _phyPos += curSize;
+    if (_phySize < _phyPos)
+      _phySize = _phyPos;
+    _cachedPos += curSize;
+    _cachedSize -= curSize;
+    size -= curSize;
+  }
+  return S_OK;
+}
+
+HRESULT CCacheOutStream::FlushCache()
+{
+  return MyWrite(_cachedSize);
+}
+
+CCacheOutStream::~CCacheOutStream()
+{
+  FlushCache();
+  if (_virtSize != _phySize)
+    _stream->SetSize(_virtSize);
+  if (_virtPos != _phyPos)
+    _stream->Seek(_virtPos, STREAM_SEEK_SET, NULL);
+  ::MidFree(_cache);
+}
+
+STDMETHODIMP CCacheOutStream::Write(const void *data, UInt32 size, UInt32 *processedSize)
+{
+  if (processedSize)
+    *processedSize = 0;
+  if (size == 0)
+    return S_OK;
+
+  UInt64 zerosStart = _virtPos;
+  if (_cachedSize != 0)
+  {
+    if (_virtPos < _cachedPos)
+    {
+      RINOK(FlushCache());
+    }
+    else
+    {
+      UInt64 cachedEnd = _cachedPos + _cachedSize;
+      if (cachedEnd < _virtPos)
+      {
+        if (cachedEnd < _phySize)
+        {
+          RINOK(FlushCache());
+        }
+        else
+          zerosStart = cachedEnd;
+      }
+    }
+  }
+
+  if (_cachedSize == 0 && _phySize < _virtPos)
+    _cachedPos = zerosStart = _phySize;
+
+  if (zerosStart != _virtPos)
+  {
+    // write zeros to [cachedEnd ... _virtPos)
+    
+    for (;;)
+    {
+      UInt64 cachedEnd = _cachedPos + _cachedSize;
+      size_t endPos = (size_t)cachedEnd & kCacheMask;
+      size_t curSize = kCacheSize - endPos;
+      if (curSize > _virtPos - cachedEnd)
+        curSize = (size_t)(_virtPos - cachedEnd);
+      if (curSize == 0)
+        break;
+      while (curSize > (kCacheSize - _cachedSize))
+      {
+        RINOK(MyWriteBlock());
+      }
+      memset(_cache + endPos, 0, curSize);
+      _cachedSize += curSize;
+    }
+  }
+
+  if (_cachedSize == 0)
+    _cachedPos = _virtPos;
+
+  size_t pos = (size_t)_virtPos & kCacheMask;
+  size = (UInt32)MyMin((size_t)size, kCacheSize - pos);
+  UInt64 cachedEnd = _cachedPos + _cachedSize;
+  if (_virtPos != cachedEnd) // _virtPos < cachedEnd
+    size = (UInt32)MyMin((size_t)size, (size_t)(cachedEnd - _virtPos));
+  else
+  {
+    // _virtPos == cachedEnd
+    if (_cachedSize == kCacheSize)
+    {
+      RINOK(MyWriteBlock());
+    }
+    size_t startPos = (size_t)_cachedPos & kCacheMask;
+    if (startPos > pos)
+      size = (UInt32)MyMin((size_t)size, (size_t)(startPos - pos));
+    _cachedSize += size;
+  }
+  memcpy(_cache + pos, data, size);
+  if (processedSize)
+    *processedSize = size;
+  _virtPos += size;
+  if (_virtSize < _virtPos)
+    _virtSize = _virtPos;
+  return S_OK;
+}
+
+STDMETHODIMP CCacheOutStream::Seek(Int64 offset, UInt32 seekOrigin, UInt64 *newPosition)
+{
+  switch(seekOrigin)
+  {
+    case STREAM_SEEK_SET: _virtPos = offset; break;
+    case STREAM_SEEK_CUR: _virtPos += offset; break;
+    case STREAM_SEEK_END: _virtPos = _virtSize + offset; break;
+    default: return STG_E_INVALIDFUNCTION;
+  }
+  if (newPosition)
+    *newPosition = _virtPos;
+  return S_OK;
+}
+
+STDMETHODIMP CCacheOutStream::SetSize(UInt64 newSize)
+{
+  _virtSize = newSize;
+  if (newSize < _phySize)
+  {
+    RINOK(_stream->SetSize(newSize));
+    _phySize = newSize;
+  }
+  if (newSize <= _cachedPos)
+  {
+    _cachedSize = 0;
+    _cachedPos = newSize;
+  }
+  if (newSize < _cachedPos + _cachedSize)
+    _cachedSize = (size_t)(newSize - _cachedPos);
+  return S_OK;
+}
+
+
+HRESULT Update(
     DECL_EXTERNAL_CODECS_LOC_VARS
     const CObjectVector<CItemEx> &inputItems,
     const CObjectVector<CUpdateItem> &updateItems,
@@ -808,39 +1029,48 @@ HRESULT Update(
     IArchiveUpdateCallback *updateCallback)
 {
   CMyComPtr<IOutStream> outStream;
-  RINOK(seqOutStream->QueryInterface(IID_IOutStream, (void **)&outStream));
-  if (!outStream)
-    return E_NOTIMPL;
-
-  CInArchiveInfo archiveInfo;
-  if(inArchive != 0)
   {
-    inArchive->GetArchiveInfo(archiveInfo);
-    if (archiveInfo.Base != 0)
+    CMyComPtr<IOutStream> outStreamReal;
+    seqOutStream->QueryInterface(IID_IOutStream, (void **)&outStreamReal);
+    if (!outStreamReal)
+      return E_NOTIMPL;
+    CCacheOutStream *cacheStream = new CCacheOutStream();
+    outStream = cacheStream;
+    if (!cacheStream->Allocate())
+      return E_OUTOFMEMORY;
+    RINOK(cacheStream->Init(outStreamReal));
+  }
+
+  if (inArchive)
+  {
+    if (inArchive->ArcInfo.Base != 0 ||
+        inArchive->ArcInfo.StartPosition != 0 ||
+        !inArchive->IsOkHeaders)
       return E_NOTIMPL;
   }
-  else
-    archiveInfo.StartPosition = 0;
   
   COutArchive outArchive;
   outArchive.Create(outStream);
-  if (archiveInfo.StartPosition > 0)
+  /*
+  if (inArchive && inArchive->ArcInfo.StartPosition > 0)
   {
     CMyComPtr<ISequentialInStream> inStream;
-    inStream.Attach(inArchive->CreateLimitedStream(0, archiveInfo.StartPosition));
+    inStream.Attach(inArchive->CreateLimitedStream(0, inArchive->ArcInfo.StartPosition));
     RINOK(CopyBlockToArchive(inStream, outArchive, NULL));
-    outArchive.MoveBasePosition(archiveInfo.StartPosition);
+    outArchive.MoveBasePosition(inArchive->ArcInfo.StartPosition);
   }
+  */
   CMyComPtr<IInStream> inStream;
-  if(inArchive != 0)
+  if (inArchive)
     inStream.Attach(inArchive->CreateStream());
 
   return Update2(
       EXTERNAL_CODECS_LOC_VARS
-      outArchive, inArchive, inStream, 
-      inputItems, updateItems, 
-      compressionMethodMode, 
-      archiveInfo.Comment, updateCallback);
+      outArchive, inArchive, inStream,
+      inputItems, updateItems,
+      compressionMethodMode,
+      inArchive ? &inArchive->ArcInfo.Comment : NULL,
+      updateCallback);
 }
 
 }}
