@@ -2,32 +2,28 @@
 
 #include "StdAfx.h"
 
-#include "WorkDir.h"
-
 #include "Common/StringConvert.h"
 #include "Common/Wildcard.h"
 
-#include "Windows/FileName.h"
 #include "Windows/FileDir.h"
+#include "Windows/FileName.h"
 
-#if _WIN32
-static inline UINT GetCurrentCodePage() 
-  { return ::AreFileApisANSI() ? CP_ACP : CP_OEMCP; } 
+#include "WorkDir.h"
 
 using namespace NWindows;
 using namespace NFile;
-using namespace NName;
 
 UString GetWorkDir(const NWorkDir::CInfo &workDirInfo, const UString &path)
 {
   NWorkDir::NMode::EEnum mode = workDirInfo.Mode;
+  #if  !defined(UNDER_CE) && defined(_WIN32)
   if (workDirInfo.ForRemovableOnly)
   {
     mode = NWorkDir::NMode::kCurrent;
     UString prefix = path.Left(3);
-    if (prefix[1] == L':' && prefix[2] == WCHAR_PATH_SEPARATOR)
+    if (prefix[1] == L':' && prefix[2] == L'\\')
     {
-      UINT driveType = GetDriveType(GetSystemString(prefix, GetCurrentCodePage()));
+      UINT driveType = GetDriveType(GetSystemString(prefix, ::AreFileApisANSI() ? CP_ACP : CP_OEMCP));
       if (driveType == DRIVE_CDROM || driveType == DRIVE_REMOVABLE)
         mode = workDirInfo.Mode;
     }
@@ -39,6 +35,7 @@ UString GetWorkDir(const NWorkDir::CInfo &workDirInfo, const UString &path)
       mode = NZipSettings::NWorkDir::NMode::kCurrent;
     */
   }
+  #endif
   switch(mode)
   {
     case NWorkDir::NMode::kCurrent:
@@ -48,22 +45,15 @@ UString GetWorkDir(const NWorkDir::CInfo &workDirInfo, const UString &path)
     case NWorkDir::NMode::kSpecified:
     {
       UString tempDir = workDirInfo.Path;
-      NormalizeDirPathPrefix(tempDir);
+      NName::NormalizeDirPathPrefix(tempDir);
       return tempDir;
     }
     default:
     {
       UString tempDir;
-      if(!NFile::NDirectory::MyGetTempPath(tempDir))
+      if (!NDirectory::MyGetTempPath(tempDir))
         throw 141717;
       return tempDir;
     }
   }
 }
-#else
-UString GetWorkDir(const NWorkDir::CInfo &workDirInfo, const UString &path)
-{
-	return L"."; // FIXME
-}
-#endif
-
