@@ -2,53 +2,39 @@
 
 #include "StdAfx.h"
 
+#if defined( _7ZIP_LARGE_PAGES)
+#include "../../../../C/Alloc.h"
+#endif
+
 #include "Common/MyInitGuid.h"
 
 #include "Common/CommandLineParser.h"
-#include "Common/MyException.h"
 #include "Common/IntToString.h"
-#include "Common/ListFileUtils.h"
-#include "Common/StdInStream.h"
+#include "Common/MyException.h"
 #include "Common/StdOutStream.h"
 #include "Common/StringConvert.h"
 #include "Common/StringToInt.h"
-#include "Common/Wildcard.h"
 
-#include "Windows/FileDir.h"
-#include "Windows/FileName.h"
-#include "Windows/Defs.h"
 #include "Windows/Error.h"
 #ifdef _WIN32
 #include "Windows/MemoryLock.h"
 #endif
 
-#include "../../IPassword.h"
-#include "../../ICoder.h"
-#include "../Common/UpdateAction.h"
-#include "../Common/Update.h"
-#include "../Common/Extract.h"
 #include "../Common/ArchiveCommandLine.h"
 #include "../Common/ExitCode.h"
+#include "../Common/Extract.h"
 #ifdef EXTERNAL_CODECS
 #include "../Common/LoadCodecs.h"
 #endif
 
-#include "../../Compress/LZMA_Alone/LzmaBenchCon.h"
-
+#include "BenchCon.h"
+#include "ExtractCallbackConsole.h"
 #include "List.h"
 #include "OpenCallbackConsole.h"
-#include "ExtractCallbackConsole.h"
 #include "UpdateCallbackConsole.h"
 
 #include "../../MyVersion.h"
 
-#if defined( _WIN32) && defined( _7ZIP_LARGE_PAGES)
-extern "C" 
-{ 
-#include "../../../../C/Alloc.h"
-}
-#endif
-
 #include "myPrivate.h"
 #include "Windows/System.h"
 
@@ -71,7 +57,7 @@ static const char *kCopyrightString = "\n7-Zip"
 " " MY_VERSION_COPYRIGHT_DATE "\n"
 "p7zip Version " P7ZIP_VERSION ;
 
-static const char *kHelpString = 
+static const char *kHelpString =
     "\nUsage: 7z"
 #ifdef _NO_CRYPTO
     "r"
@@ -101,32 +87,22 @@ static const char *kHelpString =
     "  -ax[r[-|0]]{@listfile|!wildcard}: eXclude archives\n"
     "  -bd: Disable percentage indicator\n"
     "  -i[r[-|0]]{@listfile|!wildcard}: Include filenames\n"
-    "  -m{Parameters}: set compression Method (see the manual)\n"
-#ifdef HAVE_LSTAT
-    "  -l: don't store symlinks; store the files/directories they point to\n"
-    "  CAUTION : the scanning stage can never end because of symlinks like '..'\n"
-    "            (ex:  ln -s .. ldir)\n"
-#endif
+    "  -m{Parameters}: set compression Method\n"
     "  -o{Directory}: set Output directory\n"
+    #ifndef _NO_CRYPTO
     "  -p{Password}: set Password\n"
+    #endif
     "  -r[-|0]: Recurse subdirectories\n"
-    "  (CAUTION: this flag does not do what you think, avoid using it)\n"
+    "  -scs{UTF-8 | WIN | DOS}: set charset for list files\n"
     "  -sfx[{name}]: Create SFX archive\n"
     "  -si[{name}]: read data from stdin\n"
     "  -slt: show technical information for l (List) command\n"
-    "  -so: write data to stdout (eg: "
-        "7z"
-#ifdef _NO_CRYPTO
-        "r"
-#elif EXCLUDE_COM
-        "a"
-#endif
-        " a dummy -tgzip -so Doc.txt > archive.gz)\n"
+    "  -so: write data to stdout\n"
     "  -ssc[-]: set sensitive case mode\n"
     "  -t{Type}: Set type of archive\n"
-    "  -v{Size}[b|k|m|g]: Create volumes\n"
     "  -u[-][p#][q#][r#][x#][y#][z#][!newArchiveName]: Update options\n"
-    "  -w[path]: assign Work directory. Empty path means a temporary directory\n"
+    "  -v{Size}[b|k|m|g]: Create volumes\n"
+    "  -w[{path}]: assign Work directory. Empty path means a temporary directory\n"
     "  -x[r[-|0]]]{@listfile|!wildcard}: eXclude filenames\n"
     "  -y: assume Yes on all queries\n";
 
@@ -134,7 +110,9 @@ static const char *kHelpString =
 // exception messages
 
 static const char *kEverythingIsOk = "Everything is Ok";
-static const char *kUserErrorMessage  = "Incorrect command line"; // NExitCode::kUserError
+static const char *kUserErrorMessage = "Incorrect command line";
+static const char *kNoFormats = "7-Zip cannot find the code that works with archives.";
+static const char *kUnsupportedArcTypeMessage = "Unsupported archive type";
 
 static const wchar_t *kDefaultSfxModule = L"7zCon.sfx";
 
@@ -144,19 +122,19 @@ static void ShowMessageAndThrowException(CStdOutStream &s, LPCSTR message, NExit
   throw code;
 }
 
-static void PrintHelpAndExit(CStdOutStream &s) // yyy
+static void PrintHelpAndExit(CStdOutStream &s)
 {
   s << kHelpString;
   ShowMessageAndThrowException(s, kUserErrorMessage, NExitCode::kUserError);
 }
 
 #ifndef _WIN32
-static void GetArguments(int numArguments, const char *arguments[], UStringVector &parts)
+static void GetArguments(int numArgs, const char *args[], UStringVector &parts)
 {
   parts.Clear();
-  for(int i = 0; i < numArguments; i++)
+  for (int i = 0; i < numArgs; i++)
   {
-    UString s = MultiByteToUnicodeString(arguments[i]);
+    UString s = MultiByteToUnicodeString(args[i]);
     parts.Add(s);
   }
 }
@@ -174,7 +152,7 @@ static void ShowCopyrightAndHelp(CStdOutStream &s, bool needHelp)
     if (nbcpu > 1) s << nbcpu << " CPUs)\n";
     else           s << nbcpu << " CPU)\n";
 
-  if (needHelp) 
+  if (needHelp)
     s << kHelpString;
 }
 
@@ -202,26 +180,25 @@ static inline char GetHex(Byte value)
 }
 
 int Main2(
-  #ifndef _WIN32  
-  int numArguments, const char *arguments[]
+  #ifndef _WIN32
+  int numArgs, const char *args[]
   #endif
 )
 {
-  #ifdef _WIN32  
+  #if defined(_WIN32) && !defined(UNDER_CE)
   SetFileApisToOEM();
   #endif
   
   UStringVector commandStrings;
-  #ifdef _WIN32  
+  #ifdef _WIN32
   NCommandLineParser::SplitCommandLine(GetCommandLineW(), commandStrings);
   #else
-  // GetArguments(numArguments, arguments, commandStrings);
-  extern void mySplitCommandLine(int numArguments,const char *arguments[],UStringVector &parts);
-  mySplitCommandLine(numArguments,arguments,commandStrings);
-
+  // GetArguments(numArgs, args, commandStrings);
+  extern void mySplitCommandLine(int numArgs,const char *args[],UStringVector &parts);
+  mySplitCommandLine(numArgs,args,commandStrings);
   #endif
 
-  if(commandStrings.Size() == 1)
+  if (commandStrings.Size() == 1)
   {
     ShowCopyrightAndHelp(g_StdOut, true);
     return 0;
@@ -234,17 +211,19 @@ int Main2(
 
   parser.Parse1(commandStrings, options);
 
-  if(options.HelpMode)
+  if (options.HelpMode)
   {
     ShowCopyrightAndHelp(g_StdOut, true);
     return 0;
   }
 
-  #if defined(_WIN32) && defined(_7ZIP_LARGE_PAGES)
+  #if defined(_7ZIP_LARGE_PAGES)
   if (options.LargePages)
   {
     SetLargePageSize();
+#ifdef _WIN32
     NSecurity::EnableLockMemoryPrivilege();
+#endif
   }
   #endif
 
@@ -269,6 +248,17 @@ int Main2(
     throw CSystemException(result);
 
   bool isExtractGroupCommand = options.Command.IsFromExtractGroup();
+
+  if (codecs->Formats.Size() == 0 &&
+        (isExtractGroupCommand ||
+        options.Command.CommandType == NCommandType::kList ||
+        options.Command.IsFromUpdateGroup()))
+    throw kNoFormats;
+
+  CIntVector formatIndices;
+  if (!codecs->FindFormatForArchiveType(options.ArcType, formatIndices))
+    throw kUnsupportedArcTypeMessage;
+
   if (options.Command.CommandType == NCommandType::kInfo)
   {
     stdStream << endl << "Formats:" << endl;
@@ -279,8 +269,8 @@ int Main2(
       #ifdef EXTERNAL_CODECS
       if (arc.LibIndex >= 0)
       {
-        char s[32];
-        ConvertUInt64ToString(arc.LibIndex, s);
+        char s[16];
+        ConvertUInt32ToString(arc.LibIndex, s);
         PrintString(stdStream, s, 2);
       }
       else
@@ -334,8 +324,8 @@ int Main2(
       int libIndex = codecs->GetCodecLibIndex(j);
       if (libIndex >= 0)
       {
-        char s[32];
-        ConvertUInt64ToString(libIndex, s);
+        char s[16];
+        ConvertUInt32ToString(libIndex, s);
         PrintString(stdStream, s, 2);
       }
       else
@@ -378,10 +368,17 @@ int Main2(
     }
     else
     {
-      HRESULT res = LzmaBenchCon(
-        #ifdef EXTERNAL_LZMA
-        codecs,
-        #endif
+      HRESULT res;
+      #ifdef EXTERNAL_CODECS
+      CObjectVector<CCodecInfoEx> externalCodecs;
+      res = LoadExternalCodecs(compressCodecsInfo, externalCodecs);
+      if (res != S_OK)
+        throw CSystemException(res);
+      #endif
+      res = LzmaBenchCon(
+          #ifdef EXTERNAL_CODECS
+          compressCodecsInfo, &externalCodecs,
+          #endif
         (FILE *)stdStream, options.NumIterations, options.NumThreads, options.DictionarySize);
       if (res != S_OK)
       {
@@ -396,38 +393,48 @@ int Main2(
   }
   else if (isExtractGroupCommand || options.Command.CommandType == NCommandType::kList)
   {
-    if(isExtractGroupCommand)
+    if (isExtractGroupCommand)
     {
       CExtractCallbackConsole *ecs = new CExtractCallbackConsole;
       CMyComPtr<IFolderArchiveExtractCallback> extractCallback = ecs;
 
       ecs->OutStream = &stdStream;
+
+      #ifndef _NO_CRYPTO
       ecs->PasswordIsDefined = options.PasswordEnabled;
       ecs->Password = options.Password;
+      #endif
+
       ecs->Init();
 
       COpenCallbackConsole openCallback;
       openCallback.OutStream = &stdStream;
+
+      #ifndef _NO_CRYPTO
       openCallback.PasswordIsDefined = options.PasswordEnabled;
       openCallback.Password = options.Password;
+      #endif
 
       CExtractOptions eo;
+      eo.StdInMode = options.StdInMode;
       eo.StdOutMode = options.StdOutMode;
       eo.PathMode = options.Command.GetPathMode();
       eo.TestMode = options.Command.IsTestMode();
       eo.OverwriteMode = options.OverwriteMode;
       eo.OutputDir = options.OutputDir;
       eo.YesToAll = options.YesToAll;
-      #ifdef COMPRESS_MT
+      eo.CalcCrc = options.CalcCrc;
+      #if !defined(_7ZIP_ST) && !defined(_SFX)
       eo.Properties = options.ExtractProperties;
       #endif
       UString errorMessage;
       CDecompressStat stat;
       HRESULT result = DecompressArchives(
           codecs,
-          options.ArchivePathsSorted, 
+          formatIndices,
+          options.ArchivePathsSorted,
           options.ArchivePathsFullSorted,
-          options.WildcardCensor.Pairs.Front().Head, 
+          options.WildcardCensor.Pairs.Front().Head,
           eo, &openCallback, ecs, errorMessage, stat);
       if (!errorMessage.IsEmpty())
       {
@@ -459,22 +466,33 @@ int Main2(
         stdStream << "Folders: " << stat.NumFolders << endl;
       if (stat.NumFiles != 1 || stat.NumFolders != 0)
           stdStream << "Files: " << stat.NumFiles << endl;
-      stdStream 
+      stdStream
            << "Size:       " << stat.UnpackSize << endl
            << "Compressed: " << stat.PackSize << endl;
+      if (options.CalcCrc)
+      {
+        char s[16];
+        ConvertUInt32ToHexWithZeros(stat.CrcSum, s);
+        stdStream << "CRC: " << s << endl;
+      }
     }
     else
     {
       UInt64 numErrors = 0;
       HRESULT result = ListArchives(
           codecs,
-          options.ArchivePathsSorted, 
+          formatIndices,
+          options.StdInMode,
+          options.ArchivePathsSorted,
           options.ArchivePathsFullSorted,
-          options.WildcardCensor.Pairs.Front().Head, 
-          options.EnableHeaders, 
+          options.WildcardCensor.Pairs.Front().Head,
+          options.EnableHeaders,
           options.TechMode,
-          options.PasswordEnabled, 
-          options.Password, numErrors);
+          #ifndef _NO_CRYPTO
+          options.PasswordEnabled,
+          options.Password,
+          #endif
+          numErrors);
       if (numErrors > 0)
       {
         g_StdOut << endl << "Errors: " << numErrors;
@@ -484,36 +502,39 @@ int Main2(
         throw CSystemException(result);
     }
   }
-  else if(options.Command.IsFromUpdateGroup())
+  else if (options.Command.IsFromUpdateGroup())
   {
-    UString workingDir;
-
     CUpdateOptions &uo = options.UpdateOptions;
     if (uo.SfxMode && uo.SfxModule.IsEmpty())
       uo.SfxModule = kDefaultSfxModule;
 
-    bool passwordIsDefined = 
-        options.PasswordEnabled && !options.Password.IsEmpty();
-
     COpenCallbackConsole openCallback;
     openCallback.OutStream = &stdStream;
+
+    #ifndef _NO_CRYPTO
+    bool passwordIsDefined =
+        options.PasswordEnabled && !options.Password.IsEmpty();
     openCallback.PasswordIsDefined = passwordIsDefined;
     openCallback.Password = options.Password;
+    #endif
 
     CUpdateCallbackConsole callback;
     callback.EnablePercents = options.EnablePercents;
+
+    #ifndef _NO_CRYPTO
     callback.PasswordIsDefined = passwordIsDefined;
     callback.AskPassword = options.PasswordEnabled && options.Password.IsEmpty();
     callback.Password = options.Password;
+    #endif
     callback.StdOutMode = uo.StdOutMode;
     callback.Init(&stdStream);
 
     CUpdateErrorInfo errorInfo;
 
-    if (!uo.Init(codecs, options.ArchiveName, options.ArcType))
-      throw "Unsupported archive type";
-    HRESULT result = UpdateArchive(codecs, 
-        options.WildcardCensor, uo, 
+    if (!uo.Init(codecs, formatIndices, options.ArchiveName))
+      throw kUnsupportedArcTypeMessage;
+    HRESULT result = UpdateArchive(codecs,
+        options.WildcardCensor, uo,
         errorInfo, &openCallback, &callback);
 
 #ifdef ENV_UNIX
@@ -601,7 +622,7 @@ int Main2(
     }
     return exitCode;
   }
-  else 
+  else
     PrintHelpAndExit(stdStream);
   return 0;
 }
