@@ -1,10 +1,11 @@
 // ListViewDialog.h
 
-#ifndef __LISTVIEWDIALOG_H
-#define __LISTVIEWDIALOG_H
+#ifndef __LISTVIEW_DIALOG_H
+#define __LISTVIEW_DIALOG_H
 
 #include "Windows/Control/Dialog.h"
 #include "Windows/Control/ListView.h"
+
 #include "ListViewDialogRes.h"
 
 class CListViewDialog: public NWindows::NControl::CModalDialog
@@ -19,14 +20,13 @@ class CListViewDialog: public NWindows::NControl::CModalDialog
 public:
   UString Title;
   bool DeleteIsAllowed;
-  UStringVector Strings;
   bool StringsWereChanged;
+  UStringVector Strings;
   int FocusedItemIndex;
 
   INT_PTR Create(HWND wndParent = 0) { return CModalDialog::Create(IDD_DIALOG_LISTVIEW, wndParent); }
 
   CListViewDialog(): DeleteIsAllowed(false) {}
-
 };
 
 #endif
