@@ -41,6 +41,7 @@ bool DeleteFileAlways(LPCTSTR name);
 #ifndef _UNICODE
 bool DeleteFileAlways(LPCWSTR name);
 #endif
+bool RemoveDirectoryWithSubItems(const UString &path);
 
 #ifndef _WIN32_WCE
 bool MyGetFullPathName(LPCTSTR fileName, CSysString &resultPath, 
@@ -75,6 +76,7 @@ public:
   ~CTempFile() { Remove(); }
   void DisableDeleting() { _mustBeDeleted = false; }
   UINT Create(LPCTSTR dirPath, LPCTSTR prefix, CSysString &resultPath);
+  bool Create(LPCTSTR prefix, CSysString &resultPath);
   bool Remove();
 };
 
@@ -82,6 +84,32 @@ public:
 typedef CTempFile CTempFileW;
 #endif
 
+bool CreateTempDirectory(LPCWSTR prefixChars, UString &dirName);
+
+class CTempDirectory
+{
+  bool _mustBeDeleted;
+  CSysString _tempDir;
+public:
+  const CSysString &GetPath() const { return _tempDir; }
+  CTempDirectory(): _mustBeDeleted(false) {}
+  ~CTempDirectory() { Remove();  }
+  bool Create(LPCTSTR prefix) ;
+  bool Remove()
+  {
+    if (!_mustBeDeleted)
+      return true;
+    _mustBeDeleted = !RemoveDirectoryWithSubItems(_tempDir);
+    return (!_mustBeDeleted);
+  }
+  void DisableDeleting() { _mustBeDeleted = false; }
+};
+
+#ifdef _UNICODE
+typedef CTempDirectory CTempDirectoryW;
+#endif
+
+
 }}}
 
 #endif
